/*
    \brief Hauptinclude mit den allgemeinen Funktionen und Einbindung aller Untermodule.

    \author Bernd Ottmann
    \date   21.02.2003
*/
#pragma once
#ifndef DATAFOX_API_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define DATAFOX_API_H
#endif

#include "df_defines.h"
#include "df_access_control.h"
#include "df_access_control_lists.h"
#include "df_access_control_lists_v1.h"
#include "df_active_mode.h"
#include "df_connection.h"
#include "df_defines.h"
#include "df_files.h"
#include "df_fingerprint.h"
#include "df_lists.h"
#include "df_logging.h"
#include "df_setup_lists.h"
#include "df_setup_records.h"
#include "df_system_records.h"
#include "df_timeboy_lists.h"

/*!
    \brief Prüft ob das Gerät erreichbar ist.

    Prüft durch Abruf der Firmwareversion die Erreichbarkeit des Gerätes.

    \ingroup utils

    \export{_DFCCheckAE@8, DFCCheckAE}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.

    \retval 0 Nicht Erreichbar. Gerät antwortete nicht auf die Anfragen.
    \retval 1 Erreichbar.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCCheckAE( int connectionNumber, int deviceAddress );

/*!
    \brief Prüft ob das Gerät erreichbar ist.

    Prüft mittels angebbarer Anzahl Anfragen die Erreichbarkeit des Gerätes.

    \ingroup utils

    \export{_DFCCheckDevice@16, DFCCheckDevice}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.
    \param[in] devicePollRetry Anzahl der Wiederholversuche bei älteren Protokollversionen. Ab Firmwareversionen
    größer 04.01.00 wird durch Aufruf von DFCGetVersionFirmware() die Erreichbarkeit
    des Gerätes geprüft und dieser Parameterwert ignoriert.

    \retval 0 Nicht Erreichbar. Gerät antwortete nicht auf die Anfragen.
    \retval 1 Erreichbar.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCCheckDevice( int connectionNumber, int deviceAddress, int *errorNumber, int devicePollRetry );

/*!
    \brief Setzt das übergebene Datum und die Uhrzeit in dem Gerät.

    Wird an \a dateTimeBuffer ein \a nullptr übergeben, wird die aktuelle Systemzeit verwendet.
    Ansonsten muss das zuvor aufbereitete Datum mit Uhrzeit in einem Datenarray vorliegen.

    \ingroup system

    \export{_DFCComSetTime@12, DFCComSetTime}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] dateTimeBuffer Datenarray mit Werten für [Datum und Uhrzeit](\ref dfc_type_date_time) oder ein \a nullptr
    wobei dann die aktuelle Systemzeit verwendet wird.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCComSetTime( int connectionNumber, int deviceAddress, const unsigned char *dateTimeBuffer );

/*!
    \brief Liest das Datum und die Uhrzeit aus dem Gerät.

    Das Datum mit Uhrzeit wird in einen Datenarray abgelegt.

    \ingroup system

    \export{_DFCComGetTime@12, DFCComGetTime}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] dateTimeBuffer Datenarray für die Werte von [Datum und Uhrzeit](\ref dfc_type_date_time).

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCComGetTime( int connectionNumber, int deviceAddress, unsigned char *dateTimeBuffer );

/*!
    \brief Sendet eine Meldung ggf. zur Anzeige auf einem Display.

    Die Meldung kann zur optischen Quittierung der eingegebenen Benutzerdaten im Onlinebetrieb verwendet werden.
    Da die Meldung sich in die Bedienung des Gerätes integriert, steht sie maximal bis zur nächsten Eingabe zur Verfügung.

    \ingroup system

    \export{_DFCComSendMessage@28, DFCComSendMessage}

    \note Bei Verwendung der Setupoption "Server online" nuss nach dem Abruf eine Online-Datensatzes dieser mit einer
    Meldung quittiert werden, ansonsten bekommt der Anwender am Terminal eine Feldermeldung angezeigt.
    Sie können auch eine Dummy-Meldung mit der Anzeigedauer von 0 senden um eine
    Freischaltung der Bedinung zu erreichen wobei keine Meldung am Display angezeigt wird..

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] visibleTime Anzeigedauer. (\a visibleTime * 250ms). 255 -> nicht automatisch wegblenden.
    \param[in] doAction Ist für Eingriffe in die Bedienung vorgesehen.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>0</td><td>Nachricht, wenn möglich anzeigen.</td></tr>
        <tr><td>1</td><td>Nachricht enthält zusätzliche Steuerdaten.</td></tr>
    </table>
    \param[in] audibleSignal Meldung aktustisch untermalen.
    <table class="paramvalues">
        <tr><th>Symbol</th><th>Bedeutung</th></tr>
        <tr><td>+</td><td>50 ms Ton an</td></tr>
        <tr><td>-</td><td>50 ms Ton aus</td></tr>
    </table>
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>0</td><td>- Off</td></tr>
        <tr><td>1</td><td>++++++++ Ok</td></tr>
        <tr><td>2</td><td>+++++-----+++++ Error</td></tr>
        <tr><td>3</td><td>+++++</td></tr>
        <tr><td>4</td><td>++--+++++</td></tr>
        <tr><td>5</td><td>++--++</td></tr>
        <tr><td>6</td><td>+++++--+++++</td></tr>
        <tr><td>7</td><td>++--++--++</td></tr>
        <tr><td>8</td><td>+++++--+++++--+++++</td></tr>
        <tr><td>9</td><td>++--+++++--++</td></tr>
        <tr><td>10</td><td>+++++--++--+++++</td></tr>
    </table>
    \param[in] text Zeichenfolge mit dem anzuzeigenden Text. Der Text darf maximal 242 Zeichen lang sein.
    Durch einfügen von \\r in die Zeichenfolge kann ein unterteilen in Zeilen erreicht werden.
    \param[in] textLength Laenge des anzuzeigenden Textes.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCComSendMessage( int connectionNumber, int deviceAddress, unsigned char visibleTime, 
                                                         unsigned char doAction, unsigned char audibleSignal, const char *text, int textLength );

/*!
    \brief Sendet eine Memonachricht an Geräte.

    Der gesendete Text wird in einem eigens dafür vorgesehenen Speicherbereich
    abgelegt und kann über die Pfeiltasten in den "nicht PZE-Moden" erreicht werden.

    \ingroup system

    \export{_DFCComSendInfotext@16, DFCComSendInfotext}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] text Zeichenfolge mit dem anzuzeigenden Text.
    \param[in] textLength Länge des anzuzeigenden Textes.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCComSendInfotext( int connectionNumber, int deviceAddress, const char *text, int textLength );

/*!
    \brief Die Seriennummer des Gerätes auslesen.

    \ingroup system

    \export{_DFCGetSeriennummer@16, DFCGetSeriennummer}

    \note Um eine eindeutige Geräte-Id des Datafox-Gerätes zu erhalten wird zusätzliche zu der Seriennummer noch
    der Gerätetyp benötigt. Bei Datafox sind die Seriennummernkreise an den Gerätetyp gebunden. Es können somit
    unterschiedliche Gerätetypen mit den selben Seriennummern in Ihrem System auftreten.\n
    Den Gerätetypen können Sie beispielsweise über den Namenskürzel der Firmwareversion per DFCGetVersionFirmware() ermitteln.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.
    \param[out] value Seriennummer.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetSeriennummer( int connectionNumber, int deviceAddress, int *errorNumber, int *value );

/*!
    \brief Setzen einer Rückruffunktion.

    Die Rückruffunktion wird während des Kommunikationsablaufs fortwährend
    aufgerufen und bietet die Möglichkeit eines frühzeitigen abbruches der Befehlsausführung.

    \ingroup library

    \export{_DFCSetCallBack@8, DFCSetCallBack}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] functionPointer Zeiger auf die Rückruffunktion oder nullptr für abschalten. Gibt die Funktion eine 1
    zurück wird mit der Kommunikation fortgefahren, bei Rückgabe einer 0 wird mit einem
    entsprechenden Fehler die Befehlsausführung abgebrochen.
*/
DF_API_EXTERN_DECORATE void DF_API_CALL DFCSetCallBack( int connectionNumber, int (*functionPointer)(void) );

/*!
    \brief Auflösen einer Fehlernummer in seine Textentsprechung.

    Die von den meisten Funktionen über den Parameter errorNumber zurückgegebene Fehlernummer oder mittels
    DFCGetLastErrorNumber() ermittelte, kann mit dieser Funktion in seine lesbare Bedeutung umgewandelt werden.

    \ingroup library

    \export{_DFCGetErrorText@20, DFCGetErrorText}

    \note Die hier gelieferte Textentprechung ist nicht für die Anzeige bei Ihren Kunden vorgesehen. Bitte beachten
    Sie, dass bei Ihren Kunden eigene Fehlermeldungen bei Kommunikationsstörungen angezeigt werden. Die
    Fehlermeldungen der Kommunikationsbibliothek sind allgemein gehalten und zumeist nur im Einzelbefehlskontext
    sinnvoll zu verstehen.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[out] errorNumber Fehlernummer die nach Fehlern den letzten [Fehlercode](\ref dfc_error_numbers) enthält. Siehe auch DFCGetLastErrorNumber().
    \param[in] languageId Angabe der zu verwendenden Sprache. Standardwert und aktuell einziger möglicher ist 0 = Deutsch.
    \param[out] value Zeichenarray für den Fehlertext.
    \param[in] valueLength Länge des Zeichenarrays \a value.
*/
DF_API_EXTERN_DECORATE void DF_API_CALL DFCGetErrorText( int connectionNumber, int errorNumber, int languageId, char *value, int valueLength );

/*!
    \brief Setzen des Wertes einer Setup- oder Systemvariablen.

    Die im Setup vorhandenen globalen Variablen oder im System veränderbare Variablen können über diese Funktion
    geändert werden.

    \ingroup system

    \export{_DFCSetGlobVar@24, DFCSetGlobVar}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] name Zeichenfolge mit dem Namen der Variablen. Oder im ersten Byte der Zeichenfolge steht der Index
    der anzusprechenden Setupvariablen.
    \param[in] nameType Gibt an ob name auf eine Zeichenfolge für den Namen einer Setup- oder Systemvariablen verweist oder
    ob es sich um ein Indexbyte einer Setupvariablen handelt.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>0</td><td>\c name zeigt auf ein Indexbyte. Der gültige Wertebereich ist 1 - 8.</td></tr>
        <tr><td>1</td><td>\c name zeigt auf eine Zeichenfolge mit dem Namen einer Setupvariablen. Maximal 16 Zeichen.</td></tr>
        <tr><td>2</td><td>\c name zeigt auf eine Zeichenfolge mit dem Namen einer Systemvariablen.</td></tr>
    </table>
    \param[in] value Zeichenfolge mit dem zu setzenden Wert.
     <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>\c nameType = 0 oder 1</td><td>Bis Firmware Version 04.03.00 maximal 20 Zeichen, danach maximal 60 Zeichen.</td></tr>
        <tr><td>\c nameType = 2</td><td>Die mögliche Länge ist abhängig von der Systemvariablen.</td></tr>
    </table>
   \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCSetGlobVar( int connectionNumber, int deviceAddress, const char *name, int nameType, const char *value, int *errorNumber );

/*!
    \brief Lesen des Wertes einer Setup- oder Systemvariablen.

    Die im Setup vorhandenen globalen Variablen oder im System veränderbare Variablen können über diese Funktion
    gelesen werden.

    \ingroup sytem

    \export{_DFCGetGlobVar@28, DFCGetGlobVar}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] name Zeichenfolge mit dem Namen der Variablen. Oder im ersten Byte der Zeichenfolge steht der Index
    der anzusprechenden Setupvariablen.
    \param[in] nameType Gibt an ob name auf eine Zeichenfolge für den Namen einer Setup- oder Systemvariablen verweist oder
    ob es sich um ein Indexbyte einer Setupvariablen handelt.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>0</td><td>\c name zeigt auf ein Indexbyte. Der gültige Wertebereich ist 1 - 8.</td></tr>
        <tr><td>1</td><td>\c name zeigt auf eine Zeichenfolge mit dem Namen einer Setupvariablen. Maximal 16 Zeichen.</td></tr>
        <tr><td>2</td><td>\c name zeigt auf eine Zeichenfolge mit dem Namen einer Systemvariablen.</td></tr>
    </table>
    \param[out] value Zeichenarray für den Wert.
     <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>\c nameType = 0 oder 1</td><td>Bis Firmware Version 04.03.00 maximal 20 Zeichen, danach maximal 60 Zeichen.</td></tr>
        <tr><td>\c nameType = 2</td><td>Die mögliche Länge ist abhängig von der Systemvariablen.</td></tr>
    </table>
    \param[in] valueLength Länge des Zeichenarrays \a value.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetGlobVar( int connectionNumber, int deviceAddress, const char *name, int nameType, char *value, int valueLength, int *errorNumber );

/*!
    \brief Schliessen eines Ausgangs (Relais).

    Internen digitalen Ausgang oder externen digitalen Ausgang (z.B. IO-Module)
    für eine bestimmbare Zeit schalten.

    \ingroup system

    \export{_DFCCloseRelay@20, DFCCloseRelay}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] number Nummer des Ausgangs. Bereich 0 bis 40.
    \param[in] timeout Zeit in Sekunden, für welche der Ausgang geschaltet wird.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCCloseRelay( int connectionNumber, int deviceAddress, int number, int timeout, int *errorNumber );

/*!
    \brief Abfragen des aktuellen Zustandes und der verbleibenden Zeit eines geschalteten Ausgangs (Relais).

    Der aktuelle Zustand und die verbleibende Zeit des geschlossenen Ausgangs wird abgefragt.

    \ingroup system

    \export{_DFCGetRelayState@24, DFCGetRelayState}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] number Nummer des Ausgangs. Bereich 0 bis 40.
    \param[out] state Zeiger auf eine Variable, welche den aktuellen Schaltzustand des Ausgangs erhaelt.
    \param[out] timeout Zeiger auf die Variable, welche die verbleibende Zeit in Sekunden erhaelt.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetRelayState( int connectionNumber, int deviceAddress, int number, int *state, int *timeout, int *errorNumber );

/*!
    \brief Öffnen eines Ausgangs (Relais).

    Internen digitalen Ausgang oder externen digitalen Ausgang (z.B. IO-Module) schalten.

    \ingroup system

    \export{_DFCOpenRelay@16, DFCOpenRelay}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] number Nummer des Ausgangs. Bereich 0 bis 40.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCOpenRelay( int connectionNumber, int deviceAddress, int number, int *errorNumber );

/*!
    \brief Abfrage der zuletzt mit DFCCheckDevice() benötigten Anzahl Versuche.

    Die Anzahl der benötigten Versuche können für eventuelle Statistikzwecke
    verwendet werden. Sie werden nur bei Firmwareversionen kleiner 04.01.00
    verwendet.

    \ingroup library

    \export{_DFCGetDevicePollRetry@4, DFCGetDevicePollRetry}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.

    \return Anzahl der benötigten Versuche bis Gerät als verfügbar galt.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetDevicePollRetry( int connectionNumber );

/*!
    \brief Liefert die im Gerät befindliche Firmwareversion und den Namenskürzel.

    \ingroup system

    \export{_DFCGetVersionFirmware@16, DFCGetVersionFirmware}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] value Zeichenpuffer für die Firmwareversion. Mindestgröße 64 Byte.
    <table class="paramvalues">
        <tr><th>Gerätenummer</th><th>Name</th><th>Kürzel</th></tr>
        <tr><td>255</td><td>Unbekannter Gerätetyp</td><td></td></tr>
        <tr><td>0</td><td>PZE-MasterIV</td><td>PZE</td></tr>
        <tr><td>1</td><td>ZK-MasterIV</td><td>ZK</td></tr>
        <tr><td>2</td><td>BDE-MasterIV</td><td>BDE</td></tr>
        <tr><td>3</td><td>Mobil-MasterIV</td><td>MOBIL</td></tr>
        <tr><td>4</td><td>MDE-BoxIV</td><td>MDE</td></tr>
        <tr><td>5</td><td>AE-MasterIV</td><td>AE</td></tr>
        <tr><td>6</td><td>TimeboyIV</td><td>TIMEBOY</td></tr>
        <tr><td>7</td><td>Exklusiv-Line IV</td><td>EXKLUSIV</td></tr>
        <tr><td>8</td><td>Flex-MasterIV</td><td>FLEX</td></tr>
        <tr><td>9</td><td>TimeboyIV Mobil-PZE</td><td>TB-MOBIL-PZE</td></tr>
        <tr><td>10</td><td>EVO 2.8</td><td>Evo28</td></tr>
        <tr><td>11</td><td>EVO 4.3</td><td>Evo43</td></tr>
        <tr><td>12</td><td>IPC Embedded-System</td><td>VARIO</td></tr>
        <tr><td>13</td><td>IPC Embedded-System</td><td>IPC</td></tr>
        <tr><td>14</td><td>IPC Embedded-System</td><td>Q7</td></tr>
        <tr><td>15</td><td>IO-Box V4</td><td>IOBOX</td></tr>
        <tr><td>16</td><td>KYO Inloc</td><td>ZKBOX</td></tr>
        <tr><td>17</td><td>Mobil-Box V4</td><td>MOBILBOX</td></tr>
        <tr><td>18</td><td>Dockingstation V2</td><td>DockingV2</td></tr>
        <tr><td>19</td><td>Fahrzeugdatenlogger V2</td><td>LoggerV2</td></tr>
        <tr><td>20</td><td>KYO Oneloc</td><td>ZKKNOTEN</td></tr>
        <tr><td>21</td><td>EVO 3.5 Universal</td><td>EVO_3_5_UNIVERSAL</td></tr>
        <tr><td>22</td><td>SAMS-OCL-Reader</td><td>SAMS-OCL</td></tr>
        <tr><td>23</td><td>EVO 3.5</td><td>EVO35</td></tr>
        <tr><td>24</td><td>EVO Agera</td><td>ZKREADERPLUS</td></tr>
        <tr><td>25</td><td>KYO Cenloc</td><td>25</td></tr>
        <tr><td>26</td><td>Vario 10</td><td>26</td></tr>
        <tr><td>27</td><td>Vario 12</td><td>27</td></tr>
        <tr><td>28</td><td>Vario 15</td><td>28</td></tr>
        <tr><td>29</td><td>Vario 17</td><td>29</td></tr>
        <tr><td>30</td><td>IPC EVO 12.1</td><td>30</td></tr>
        <tr><td>31</td><td>IPC EVO 15</td><td>31</td></tr>
        <tr><td>32</td><td>IPC EVO 18.5</td><td>32</td></tr>
        <tr><td>33</td><td>IPC EVO 24</td><td>33</td></tr>
        <tr><td>34</td><td>EVO Intera 2</td><td>34</td></tr>
        <tr><td>35</td><td>EVO 4.6 FlexKey</td><td>35</td></tr>
        <tr><td>36</td><td>EVO 5.0 Pure</td><td>35</td></tr>
    </table>
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetVersionFirmware( int connectionNumber, int deviceAddress, char *value, int *errorNumber );

#ifndef DOXYGEN_SHOULD_SKIP_THIS
// \export{_DFCGetVersionFirmwareFromFile@16, DFCGetVersionFirmwareFromFile}
DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetVersionFirmwareFromFile( int connectionNumber, const char *filePath, char *value, int *errorNumber );

/*!
    \brief Die Hardware Info vom Gerät lesen.

    Liest die Hardware Info aus dem Gerät und schreibt sie in eine anzugebende Ini-Datei.

    \ingroup system

    \export{_DFCReadHardwareInfo@24, DFCReadHardwareInfo}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] value Zeiger auf den String fuer die Hardwareinfo.
    \param[in] valueLength Zeiger auf die Laengenangabe des Strings.
    \param[in] flags Zusatzangaben. (noch unverwendet)
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCReadHardwareInfo( int connectionNumber, int deviceAddress, char *value, 
                                                           int *valueLength, int flags, int *errorNumber );
#endif


#if (DF_API_VERSION >= 0x04010600)
/*!
    \brief Setzt benötigte Zusatzoptionen der Firmware.

    Über die Optionen können Sie zum einen Steuern was die Firmware an Funktionalitäten unterstützten soll. Hierbei wird vor 
    der Übertragung einer Firmwaredatei geprüft ob die benötigten Funktionen durch diese unterstützt werden. 
    Zum anderen geben Sie über die Optionen ein gewisses Verhalten vor. Hierdurch können Sie zum Beispiel angeben, dass ein bestehendes 
    Setup und die Listen nach einem Firmwareupdate beibehalten werden sollen.

    \ingroup system

    \export{_DFCSetOptionFirmware@20, DFCSetOptionFirmware}

    \deprecated Bitte beachten Sie, dass die Optionen 1 - 8 eine Firmwareversion kleiner 04.03.00 voraussetzen. 
    Diese Optionen werden nicht durch die Firm- und Softwareversion 04.03.xx beachtet. 
    Die Option "Setup und Listen beibehalten" wird weiterhin unterstützt.

    \note Die Option "Setup und Listen beibehalten" wird in der Version 04.03.xx voll unterstützt. Sie wird auserdem noch 
    ab der Mobil-MasterIV-Version 04.01.04.32 unterstützt, hierbei muss beim Update mindestens die Firmwareversion 
    04.01.04.59 übertragen werden.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] mask Wert der anzusprechenden Optionen.
    <table class="paramvalues">
        <tr><th>Bitnummer</th><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>1</td><td>Serie der externen Zutrittsleser. Wertbedeutung der Option 0 = GIS, 1 = PHG</td></tr>
        <tr><td>2</td><td>2</td><td>Verwendung eines Datafox IO-Modul für MDE.</td></tr>
        <tr><td>3</td><td>4</td><td>Verwendung angebundener Timeboys.</td></tr>
        <tr><td>4</td><td>8</td><td>Unterstützung der Com-Server Funktionalität über DLL-Funktionen vorhanden.</td></tr>
        <tr><td>5</td><td>16</td><td>Transpondermenü zum Programmieren der Ausweise am Terminal vorhanden.</td></tr>
        <tr><td>6</td><td>32</td><td>Update des Biokey-Moduls über DLL-Funktion vorhanden.</td></tr>
        <tr><td>7</td><td>64</td><td>Unterstützung des Http - Protokolls vorhanden.</td></tr>
        <tr><td>8</td><td>128</td><td>Unterstüztung der Signalverarbeitung (digitaler Eingänge) vorhanden.</td></tr>
        <tr><td>9</td><td>256</td><td>Vorhandenes Setup und Listen werden übernommen und durch ein Firmwareupdate nicht gelöscht.</td></tr>
        <tr><td>...</td><td></td><td>Nicht verwendet. Bitte immer 0 übergeben.</td></tr>
    </table>
     Sie geben durch die gesetzten Bits an, welche Optionen bei der Ausführung beachtet werden.\n
     Als Beispiel geben Sie für das Ansprechen der Optionen "Transpondermenü" und "Setup und Listen beibehalten" (16 + 256 =) 272 als Wert an.
    \param[in] options Anzunehmender Zustand der gewählten Option.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>0</td><td>Aus</td></tr>
        <tr><td>1</td><td>Ein</td></tr>
    </table>
    Der Zustand der Bits gibt an ob die Option aktiviert oder deaktiviert werden soll.
    Als Beispiel geben Sie für die obige Beispielmaske, für das Deaktivieren der Optionen "Transpondermenü" 
    und Aktivieren der Option "Setup und Listen beibehalten" (0 + 256 =) 256 als Wert an. Alle anderen Bits, werden durch die angegebene Maske ignoriert.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.04 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCSetOptionFirmware( int connectionNumber, int deviceAddress, int mask, int options, int *errorNumber );

/*!
    \brief Liest die Werte der Zusatzoptionen zur Firmware.

    Über die Optionen wird zum einen gesteuern was die Firmware an Funktionalitäten unterstützten soll. Zum anderen wird über die Optionen 
    ein gewisses Verhalten vorgegeben.

    \ingroup system

    \export{_DFCGetOptionFirmware@20, DFCGetOptionFirmware}

    \deprecated Bitte beachten Sie, dass die Optionen 1 - 8 eine Firmwareversion kleiner 04.03.00 voraussetzen.
    Diese Optionen werden nicht durch die Firm- und Softwareversion 04.03.xx beachtet.
    Die Option "Setup und Listen beibehalten" wird weiterhin unterstützt.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] mask Wert der angesprochenen Optionen.
    <table class="paramvalues">
        <tr><th>Bitnummer</th><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>1</td><td>Serie der externen Zutrittsleser. Wertbedeutung der Option 0 = GIS, 1 = PHG</td></tr>
        <tr><td>2</td><td>2</td><td>Verwendung eines Datafox IO-Modul für MDE.</td></tr>
        <tr><td>3</td><td>4</td><td>Verwendung angebundener Timeboys.</td></tr>
        <tr><td>4</td><td>8</td><td>Unterstützung der Com-Server Funktionalität über DLL-Funktionen vorhanden.</td></tr>
        <tr><td>5</td><td>16</td><td>Transpondermenü zum Programmieren der Ausweise am Terminal vorhanden.</td></tr>
        <tr><td>6</td><td>32</td><td>Update des Biokey-Moduls über DLL-Funktion vorhanden.</td></tr>
        <tr><td>7</td><td>64</td><td>Unterstützung des Http - Protokolls vorhanden.</td></tr>
        <tr><td>8</td><td>128</td><td>Unterstüztung der Signalverarbeitung (digitaler Eingänge) vorhanden.</td></tr>
        <tr><td>9</td><td>256</td><td>Vorhandenes Setup und Listen werden übernommen und durch ein Firmwareupdate nicht gelöscht.</td></tr>
        <tr><td>...</td><td></td><td>Nicht verwendet. Bitte immer 0 übergeben.</td></tr>
    </table>
     Die Maske gibt an, welche Werte ausgelesen wurden und somit über den Parameter \c options zur verfügung stehen.\n
    \param[out] options Aktueller Zustand der Option.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>0</td><td>Aus</td></tr>
        <tr><td>1</td><td>Ein</td></tr>
    </table>
    Der Zustand der Maske in Kombination der Bits gibt an, ob die Option aktiviert oder deaktiviert ist.
    \code{.c}
    // Bitnummer 9 entspricht Wert 256 oder 0x100 hex.
    if ( (mask & 0x100) == 0x100 )
    {
        if ( (options & 0x100) == 0x100 )
        {
            // Option "Setup und Listen beibehalten" aktiv.
        }
        else
        {
            // Option "Setup und Listen beibehalten" deaktiv.
        }
    }
    else
    {
        // Option nicht verfügbar.
    }
    \endcode
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.04 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetOptionFirmware( int connectionNumber, int deviceAddress, int *mask, int *options, int *errorNumber );

#endif


#if (DF_API_VERSION >= 0x04010700)

/*!
    \brief Setzt den zu verwendenden Schrifttyp zur Darstellung der Nachrichten.

    Setzt den zu verwendenden Schrifttyp zur Darstellung der Nachrichten bei DFCComSendMessage()
    oder DFCComSendInfotext(). Bei den Nachrichten wird der zuvor gewählte Schrifttyp bis zum Neustart des
    Gerätes beibehalten. Sie können unterschiedliche Schrifttypen verwenden, wenn Sie jeweils vor SendMessage und
    SendInfotext einen entsprechenden Schrifttyp einstellen, der zuletzt gesetzte Schrifttyp wird für die Funktion
    solange beibehalten bis ein Neustart erfolgt.

    \ingroup system

    \export{_DFCSetFontType@16, DFCSetFontType}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] type Typ der zu vewendenden Schriftart.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>0</td><td>Standardschrift</td></tr>
        <tr><td>1</td><td>16 Pixel (7 Anzeigezeilen)</td></tr>
        <tr><td>2</td><td>16 Pixel, feste breite (7 Anzeigezeilen)</td></tr>
        <tr><td>3</td><td>19 Pixel (6 Anzeigezeilen)</td></tr>
        <tr><td>4</td><td>19 Pixel, feste breite (6 Anzeigezeilen)</td></tr>
        <tr><td>5</td><td>21 Pixel (5 Anzeigezeilen)</td></tr>
        <tr><td>6</td><td>21 Pixel, feste breite (5 Anzeigezeilen)</td></tr>
    </table>
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.07 enthalten.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCSetFontType( int connectionNumber, int deviceAddress, int type, int *errorNumber );

/*!
    \brief Über diese Funktion können Sie einen Tastendruck auslösen.

    Über diese Funktion können Sie einen Tastendruck auslösen. Sie können somit per DLL-Befehl zwischen den
    verschiedenen Funktionstasten umschalten. Es ist über diesen Befehl möglich, Tasten anzusprechen die nicht
    direkt auf der Tastatur verfügbar sind z. B. F6 - F15 beim PZE-MasterIV.

    \ingroup system

    \export{_DFCPressVirtualKey@20, DFCPressVirtualKey}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] keyCode Gibt den Tastencode an.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>112</td><td>F1 -> Entspricht einem drücken der Funktionstaste 1.</td></tr>
        <tr><td>113</td><td>F2 -> Entspricht einem drücken der Funktionstaste 2.</td></tr>
        <tr><td>114</td><td>F3 -> Entspricht einem drücken der Funktionstaste 3.</td></tr>
        <tr><td>115</td><td>F4 -> Entspricht einem drücken der Funktionstaste 4.</td></tr>
        <tr><td>116</td><td>F5 -> Entspricht einem drücken der Funktionstaste 5.</td></tr>
        <tr><td>117</td><td>F6 -> Entspricht einem drücken der Funktionstaste 6.</td></tr>
        <tr><td>118</td><td>F7 -> Entspricht einem drücken der Funktionstaste 7.</td></tr>
        <tr><td>119</td><td>F8 -> Entspricht einem drücken der Funktionstaste 8.</td></tr>
        <tr><td>120</td><td>F9 -> Entspricht einem drücken der Funktionstaste 9.</td></tr>
        <tr><td>121</td><td>F10 -> Entspricht einem drücken der Funktionstaste 10.</td></tr>
        <tr><td>122</td><td>F11 -> Entspricht einem drücken der Funktionstaste 11.</td></tr>
        <tr><td>123</td><td>F12 -> Entspricht einem drücken der Funktionstaste 12.</td></tr>
        <tr><td>124</td><td>F13 -> Entspricht einem drücken der Funktionstaste 13.</td></tr>
        <tr><td>125</td><td>F14 -> Entspricht einem drücken der Funktionstaste 14.</td></tr>
        <tr><td>126</td><td>F15 -> Entspricht einem drücken der Funktionstaste 15.</td></tr>
    </table>
    \param[in] flags Gibt zusatzliche Bedeutungen zum Tastendruck an:
    <table class="paramvalues">
        <tr><th>Bitnummer</th><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>1</td><td>Wenn Sie dieses Bit setzen, wird der Tastendruck nur ausgeführt,
            wenn sich das Gerät aktuell im Hauptmenü befindet. Wenn Sie das
            Bit nicht gesetzt haben, verhält sich das Gerät so als ob eine
            Person direkt die Taste auf der Tastatur betätigt.</td></tr>
        <tr><td>...</td><td>...</td><td>Reserviert</td></tr>
    </table>
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.07 enthalten.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCPressVirtualKey( int connectionNumber, int deviceAddress, int keyCode, int flags, int *errorNumber );

#ifndef DOXYGEN_SHOULD_SKIP_THIS
/*!
    \brief



    \ingroup system

    \export{_DFCGetPasswordKey@20, DFCGetPasswordKey}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] number Nummer des Ausgangs.
    \param[out] state Zeiger auf eine Variable, welche den aktuellen Schaltzustand des Ausgangs erhaelt.
    \param[out] timeout Zeiger auf die Variable, welche die verbleibende Zeit in Sekunden erhaelt.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.07 enthalten.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetPasswordKey( int connectionNumber, int deviceAddress, char *value, int *valueLength, int *errorNumber );
#endif

/*!
    \brief Setzen des zu verwendenden Setuppasswortes für diese Verbindung.

    Setzen des zu verwendenden Setuppasswortes für diese Verbindung. Das Passwort wird der Firmware,
    bei Aufforderung mitgeteilt. Es wird über die Setupdaten in das Gerät übertragen und muss nach der erfolgeichen
    Übertragung durch diese Routine der Bibliothek mitgeteilt werden.

    \ingroup system

    \export{_DFCSetPassword@12, DFCSetPassword}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] value Zeichenfolge des Passworts. Maximal 16 Zeichen.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.07 enthalten.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCSetPassword( int connectionNumber, const char *value, int *errorNumber );

/*!
    \brief Reset je nach gewähltem Modus durchführen.

    Reset je nach gewähltem Modus durchführen. Das Geräte macht nach der Ausführung einen Neustart.

    \ingroup system

    \export{_DFCReset@16, DFCReset}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] mode Modus, welche angibt was zurückgesetzt wird.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>Warmstart des Gerätes.</td></tr>
        <tr><td>2</td><td>Flash Formatieren. (Setup und Listen müssen neu aufgespielt werden!)</td></tr>
        <tr><td>3</td><td>Setup löschen. (Setup und Listen müssen neu aufgespielt werden!)</td></tr>
    </table>
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.07 enthalten.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCReset( int connectionNumber, int deviceAddress, int mode, int *errorNumber );

#ifndef DOXYGEN_SHOULD_SKIP_THIS
/*!
    \brief



    \ingroup system

    \export{_DFCGetFlashStatus@16, DFCGetFlashStatus}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] value Flashstatus.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.07 enthalten.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetFlashStatus( int connectionNumber, int deviceAddress, int *value, int *errorNumber );
#endif

#endif

#if (DF_API_VERSION >= 0x04020300)

/*!
    \brief Setzen des zu verwendenden Verschlüsselungspasswortes der Kommunikation.

    Setzen des zu verwendenden Verschlüsselungspasswortes der Kommunikation. Das Passwort
    wird für die Verschlüsselte Kommuniation verwendet und muss ebenfalls im Gerät hinterlegt sein.
    Siehe Systemvariable COM.KEY. Je nach Angabe des sessionTimeouts wird eine Neuaushandlung eines Sessionkeys vorgenommen.

    \ingroup system

    \export{_DFCSetCommunicationPassword@20, DFCSetCommunicationPassword}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] value Zeichenfolge des Passwortes. Maximal 16 Zeichen.
    \param[in] valueLength Länge des in \a value stehenden Passwortes.
    \param[in] sessionTimeout Gibt einen Timeout an, nachdem der ausgehandelte Session-Key seine Gültigkeit verliert
    und eine Neuaushandlung notwendig macht. Einheit ist Sekunden, bei Verwendung von 0
    wird keine Neuaushandlung durchgeführt. Wertebereich (0, 60 - 100000).
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.02.03 enthalten.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCSetCommunicationPassword( int connectionNumber, const char *value, int valueLength, int sessionTimeout, int *errorNumber );

#endif

//    \export{_DFCRecordVolume@24, DFCRecordVolume}
DF_API_EXTERN_DECORATE int DF_API_CALL DFCRecordVolume( int connectionNumber, int deviceAddress, int *recordCount, int *memoryUsage, int *memorySize, int *errorNumber );

#if (DF_API_VERSION >= 0x04010400)

/*!
    \brief Lesen der Information eines bestimmten Moduls.

    Die zurückgelieferte Information liegt in einer Zeichenfolge vor, die je nach abgefragtem Modul verschiedene
    Informationen enthalten kann. Diese Informationszeichenfolge muss für die weitere Verarbeitung ggf. in ihre
    Bestandteile zerlegt werden, wozu alle einzelnen Informationsteile durch ein CR (dezimal 13) Zeichen voneinander
    getrennt werden. Bitte beachten Sie auch, dass sich der Name aus zwei Bestandteilen zusammensetzt, zum einen aus
    Bereichsbezeichner wie ACCESS um das Zutrittsmodul anzusprechen und zum zweiten aus der Bezeichnung der
    anzusprechenden "Komponente" in diesem Bereich.\n
    Möchten Sie z. B. die Information zum angeschlossenen Stichleser am Türmodul mit der Busnummer 2 abrufen,
    benötigen Sie zum einen den "name" "ACCESS.MODULE\0" und die zugehörige "params" Parameterliste "021\0".
    Nach erfolgreicher Ausführung der Funktion stehen Ihnen die gelieferten Informationen in der Zeichenkette
    auf die "info" verweist zur Verfügung. (Das Zeichen '\0' stellt die Nullterminierung der Zeichenfolge dar.)

    \ingroup system

    \export{_DFCGetInfo@28, DFCGetInfo}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] name Name der abzurufenden Information. (z.B. "ACCESS.MODULE")
    Der Name setzt sich aus zwei Bestandteilen zusammen die mit einem Punkt miteinander verbunden werden.
    Der erste Teil gibt den Bereich an und kann folgende Zeichenfolge sein:
    <table class="paramvalues">
        <tr><th>Bereich</th><th>Bedeutung</th></tr>
        <tr><td>ACCESS</td><td>für Zutritt</td></tr>
        <tr><td>FINGERPRINT</td><td>für Fingerprint</td></tr>
        <tr><td>USBHOST</td><td>für USB-Host</td></tr>
    </table>
    der zweite Teil gibt den Namen der anzusprechenden Komponende an, und kann folgende Zeichenfolgen sein:
    <table class="paramvalues">
        <tr><th>Bereich</th><th>Komponende</th><th>Bedeutung</th></tr>
        <tr><td>ACCESS</td><td>MODULE</td><td>Zutrittsmodul / -leser</td></tr>
        <tr><td>FINGERPRINT</td><td>MODULE</td><td>Fingerprintmodul Firmwareversion</td></tr>
        <tr><td>USBHOST</td><td>MODULE</td><td>USB-Host Firmwareversion (z.B. "3.62-000VDAP")</td></tr>
    </table>
    \param[in] params Zeiger auf die ggf. anzugebende Parameterliste als Zeichenfolge. Die einzelnen Parameter
    müssen durch ein CR (dezimal 13) voneinander getrennt werden.\n
    Die anzugebende Parameterliste setzt sich für die entsprechenden Namen folgendermaßen zusammen:
    <table class="paramvalues">
        <tr><th>Bereich</th><th>Parameter</th><th>Bedeutung</th></tr>
        <tr><td>ACCESS.MODULE</td><td>1</td><td>Türmodulnummer gemäß der Spalte "TM" der Zutrittsliste "Reader". Der Wert muss immer 3 Ziffern umfassen. Genauere Angaben hierzu finden sie im Handbuch im Kapitel "Zutrittskontrolle 2".</td></tr>
        <tr><td>ACCESS.MODULE</td><td>2</td><td>Optional die Bus-Id bei Systemen mit mehreren Zutrittsbussen. Bei fehlen, wird der erste Bus angenommen.</td></tr>
        <tr><td>FINGERPRINT.MODULE</td><td>1</td><td>Leer</td></tr>
        <tr><td>USBHOST.MODULE</td><td>1</td><td>Leer</td></tr>
    </table>
    \param[out] text Zeichenarray für den Informationsstring. Die einzelnen Teilangaben sind mit dem Steuerzeichen CR ('\\r')  voneinander getrennt.\n
    Der Informationsstring für ACCESS.MODULE gliedert sich wie folgt.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Komponende</th><th>Bedeutung</th></tr>
        <tr><td>Status</td><td>1</td><td>Gibt den aktuellen Status des Moduls zurück. Genauere Angaben hierzu finden sie im Handbuch im Unterkapitel "Statusmeldungen der Zutrittskontrolle" der Zutrittskontrolle 2.</td></tr>
        <tr><td>Name</td><td>2</td><td>Gibt die Bezeichnung des angesprochenen Moduls zurück.</td></tr>
        <tr><td>Firmwareversion</td><td>3</td><td>Gibt die Firmwareversion des angesprochenen Moduls zurück.</td></tr>
    </table>
    \param[in,out] textLength Als \c in die Länge des Puffers \a text, als \c out die Länge der übergebenen Zeichenfolge.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.04 enthalten.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetInfo( int connectionNumber, int deviceAddress, const char *name, const char *params, 
                                                  char *text, int *textLength, int *errorNumber );

#ifndef DOXYGEN_SHOULD_SKIP_THIS
// \export{_DFCOpenComServerMode@40, DFCOpenComServerMode}
DF_API_EXTERN_DECORATE int DF_API_CALL DFCOpenComServerMode( int connectionNumber, int deviceAddress, const char *comm,
                                                            int baudRate, int byteSize, int parity, int stopBits, int flags, int timeoutToClose, int *errorNumber );
// \export{_DFCCloseComServerMode@12, DFCCloseComServerMode}
DF_API_EXTERN_DECORATE int DF_API_CALL DFCCloseComServerMode( int connectionNumber, int deviceAddress, int *errorNumber );
#endif

#endif

// ########################################################
// ########### FUNKTIONEN FUER ERWEITERUNGEN ##############
// ########################################################

#if (DF_API_VERSION >= 0x04030001)

/*!
    \brief Ermitteln der Libraryversion.

    Die Version wird im Format XX.XX.XX.XX geliefert.

    \ingroup library

    \export{_DFCVersionLibrary@8, DFCVersionLibrary}

    \param[out] value Zeiger auf Zeichenfolgenpuffer für die Version.
    \param[in] valueLength Länge des reservierten Puffers für die Zeichenfolge. Muss mindestens 12 sein.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.03.00 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCVersionLibrary( char *value, int valueLength );

#endif

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#if (DF_API_VERSION >= 0x04030500)
// \export{_DFCStartTransparencyMode@32, DFCStartTransparencyMode}
DF_API_EXTERN_DECORATE int DF_API_CALL DFCStartTransparencyMode( int connectionNumber, int deviceAddress, int id, int flags,
    unsigned int sizeBufferRead, unsigned int sizeBufferWrite, int timeout, int *errorNumber );

// \export{_DFCStopTransparencyMode@12, DFCStopTransparencyMode}
DF_API_EXTERN_DECORATE int DF_API_CALL DFCStopTransparencyMode( int connectionNumber, int deviceAddress, int *errorNumber );

// \export{_DFCIsTransparencyMode@4, DFCIsTransparencyMode}
DF_API_EXTERN_DECORATE int DF_API_CALL DFCIsTransparencyMode( int connectionNumber );

#endif


#if (DF_API_VERSION >= 0x04030600)
// \export{_DFCCustomCommand@36, DFCCustomCommand}
DF_API_EXTERN_DECORATE int DF_API_CALL DFCCustomCommand( int connectionNumber, int deviceAddress, int identification, int customCommand, const char *dataIn, int sizeIn, char *dataOut, int *sizeOut, int *errorNumber );

#endif
#endif

#if (DF_API_VERSION >= 0x04030800)

/*!
    \brief Liefert die Fehlernummer welche zuletzt aufgetreten ist.

    \ingroup library

    \export{DFCGetLastErrorNumber, DFCGetLastErrorNumber}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.

    \return Fehlernummer. Siehe [Tabelle der Fehlernummern](\ref dfc_error_numbers).

    \since 04.03.08 enthalten
*/
DF_API_EXTERN int DF_API_CALL DFCGetLastErrorNumber( int connectionNumber, int deviceAddress );

/*!
    \brief Setzt für die entsprechende Geräteadresse oder die gesamte Verbindungsnummer eine Unterbrechungszeit.


    \ingroup library

    \export{DFCBlockTransferSetDuration, DFCBlockTransferSetDuration}

    \note Der Blocktransfer ist für die Funktionen DFCLoadListen() und DFCLoadEntrance2List() nutzbar.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] blockTypesAllowed Gibt an welche Blockübertragungen.
    <table class="paramvalues">
        <tr><th>Bitnummer</th><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>1</td><td>Setuplisten, unterbricht DFCLoadListen().</td></tr>
        <tr><td>2</td><td>2</td><td>Zutrittslisten, unterbricht DFCLoadEntrance2List().</td></tr>
    </table>
    Durch ein Aufaddieren dieser Werte können Sie bestimmen welche Datenblöcke durch die
    angegebene Zeit unterbrochen werden sollen. Möchten Sie z. B. die Setup- und
    Zutrittslisten mit Unterbrechungen übertragen übergeben Sie eine 3.
    \param[in] duration Zeitangabe in Sekunden, nach welcher eine laufende Übertragung unterbrochen werden soll.
    Die Funktion kommt in diesem Fall mit dem Rückgabewert 0 und liefert eine entsprechende Fehlernummer.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.03.08 enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCBlockTransferSetDuration( int connectionNumber, int deviceAddress, int blockTypesAllowed, int duration );

/*!
    \brief Setzt eine bestehende Blockübertragung fort.



    \ingroup library

    \export{DFCBlockTransferResume, DFCBlockTransferResume}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.03.08 enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCBlockTransferResume( int connectionNumber, int deviceAddress );

/*!
    \brief Ruft den Typen und Zustand der ggf. vorhandenen Blockübertragung ab.


    \ingroup library

    \export{DFCBlockTransferGetState, DFCBlockTransferGetState}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] currentType Aktueller Blocktyp. Bei 0 ist keine Blockübertragung aktiv.
    <table class="paramvalues">
        <tr><th>Bitnummer</th><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>1</td><td>Setuplisten, unterbricht DFCLoadListen().</td></tr>
        <tr><td>2</td><td>2</td><td>Zutrittslisten, unterbricht DFCLoadEntrance2List().</td></tr>
    </table>
    \param[out] currentState Aktueller Zustand.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>0</td><td>Keine unterbrochene Blockübertragung aktiv.</td></tr>
        <tr><td>2</td><td>Die durch \a currentType angegebene Blockübertragung wartet auf ihre Fortsetzung.</td></tr>
    </table>

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.03.08 enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCBlockTransferGetState( int connectionNumber, int deviceAddress, int *currentType, int *currentState );

/*!
    \brief Verwirft eine aktuell laufende Blockübertragung. Die bis dahin übertragenen Daten werden verforfen.


    \ingroup library

    \export{DFCBlockTransferDiscard, DFCBlockTransferDiscard}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.03.08 enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCBlockTransferDiscard( int connectionNumber, int deviceAddress );

#endif

#if (DF_API_VERSION >= 0x04030C00)
/*!
    \brief Lesen eines Datenblocks. Dieses kann der Wert einer Systemvariablen oder der Inhalt einer Datei sein.

    Diese Funktion soll die Einschränkungen der Funktionen DFCGetGlobVar() und DFCFileDownload() aufheben und beide vereinen. 
    Die Funktion DFCGetGlobVar() hat die Einschränkung einer maximal mögliche Länge des Wertes und die 
    Funktion DFCFileDownload() bietet nicht die Möglichkeit die Daten der Datei in einem Datenpuffer zu erhalten.

    \ingroup library

    \export{DFCReadDataBlock, DFCReadDataBlock}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] name Der Name des zu lesenden Datenblocks. Dieses kann der Name einer 
                    Systemvariablen oder mit '/' beginnend, der Name einer Datei sein.
    \param[out] data Zeiger auf den Datenpuffer der die Daten erhält. Bei Verwendung des \c flags 1 wird hier ggf. der Dateiname abgelegt.
    \param[in,out] length Als \c in die reservierte Länge des auf \c data zeigenden Datenarrays, 
                          als \c out die nach \c data geschriebene Anzahl Bytes.
    \param[in] flags Flagfeld mit folgenden Bedeutungen.
    <table class="paramvalues">
        <tr><th>Bitnummer</th><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>1</td><td>Sitzt dieses Flag wird eine Datei im Tempverzeichnis erstellt, falls die angegebene 
                                    Puffergröße nicht ausreicht um die ausgelesenen Daten zu fassen. Der Rückgabewert der Funktion
                                    ist in diesem Fall 2 und in data wird der Dateiname abgelegt. Sie sind anschließend für das
                                    Löschen der Datei verantwortlich.</td></tr>
        <tr><td>...</td><td>...</td><td>Reserviert</td></tr>
    </table>

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich, Daten liegen in \c data vor.
    \retval 2 Ausführung war erfolgreich, \c data beinhaltet den Namen der Temporär erzeugten Datei mit dem Dateninhalt.

    \since 04.03.12 enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCReadDataBlock( int connectionNumber, int deviceAddress, const char *name, char *data, int *length, int flags );
#endif

// ########################################################
// #############* PRIVATE FUNKTIONEN* #################
// ########################################################

// Funktionen sind nicht dokumentiert.
// Sie werden von AESetup oder Talk verwendet.
// Unterliegen evntl. Aenderungen.

#if defined( USE_PRIVATE )

#if defined( _MSC_VER ) && defined( DF_API_EXPORT )

// \export{DllGetVersion, DllGetVersion}
DF_API_EXTERN HRESULT DF_API_CALL DllGetVersion( DLLVERSIONINFO *pdvi );

#endif

// \export{_DFCComVersion@12, DFCComVersion}
DF_API_EXTERN_DECORATE int DF_API_CALL DFCComVersion( int connectionNumber, int deviceAddress, int *errorNumber );

// \export{_DFCGetClass@8, DFCGetClass}
DF_API_EXTERN_DECORATE void* DF_API_CALL DFCGetClass( int connectionNumber, int classId );

/*
    \brief Ausführen eines Prüfplatzkommandos.

    Über diese Funktion werden der Firmware auszuführende Kommandos gesendet und die Resultate als Ergebnis geliefert.
    Die sendbare Anfrage und zu erwartende Antwort hängt von der implementierten Routine auf Firmwareseite ab.

    \ingroup library

    \export{_DFCCommandInspectionStation@36, DFCCommandInspectionStation}

    \note Zu diesem Befehl muss ein entsprechendes zusätzliches Dokument beachtet werden in welchem die übermittelten Daten
          beschrieben sind.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] requestAddress Ein Wert zwischen 0 - 255, der eine Befehlsadresse darstellt.
    \param[in] requestParams Eine zu sendende Zeichenfolge, kann auch ein Bytearray sein.
    \param[in] requestParamsSize Länge des auf \c requestParams verweisenden Puffers.
    \param[out] answerAddress Ein Wert zwischen 0 - 255, der die Antwortadresse darstellt.
    \param[out] answerParams Eine zu empfangende Zeichenfolge, kann auch ein Bytearray sein.
    \param[in, out] answerParamsSize Als Eingabe die maximal hinter \c answerParams zur verfügbung stehenden Pufferlänge, 
                    als Ausgabe die nach \c answerParams kopierte Anzahl Zeichen.
    \param[in] secondsToWaitForAnswer Angabe in Sekunden, wie lange maximal auf die Antwort gewartet wird.

    \returns Liefert 0 bei Erfolgreicher ausführung, ansonsten den auszuwertenden Fehlercode.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCCommandInspectionStation( int connectionNumber, int deviceAddress, int requestAddress,
    char *requestParams, int requestParamsSize, int *answerAddress, char *answerParams, int *answerParamsSize, int secondsToWaitForAnswer );

// \export{_DFCPing@16, DFCPing}
DF_API_EXTERN_DECORATE int DF_API_CALL DFCPing( int connectionNumber, int deviceAddress, int attempts, int *errorNumber );

#endif // USE_PRIVATE

#endif


