// file_studio_setup.h: Schnittstelle für die Klasse FileAES.
//
//////////////////////////////////////////////////////////////////////
#pragma once
#ifndef FILE_STUDIO_SETUP_H
#define FILE_STUDIO_SETUP_H

#include "interface/studio_setup_structs.h"
#include "common/crc.h"
#include "common/md5.h"
#include "common/software_version.h"

#include <cstdio>
#include <array>
#include <vector>
#include <string>
#include <bitset>
#include <cstdint>

// Angabe ueber die maximal bekannte Firmware-Version.
// Aktuell: 04.01.09.99, 04.02.06.99, 04.03.18.99
// Siehe Function: FileAES::IsSupportedFirmwareVersion

//#define FILESTUDIOHEADER_CHECK 0xDFDFDFDF ///< StructFileStudioHeader.check: Pruefwert fuer Gueltigkeit (Versionskennung) 
// Einen Dezimalwert 99 in Hexwert 0x99 wandeln.
//#define AESHEADER_DEC2VERSION(x) (x + (x / 10) * 6)
// Einen Hexwert 0x99 in Dezimalwert 99 wandeln.
//#define AESHEADER_VERSION2DEC(x) (x - (x / 16) * 6)
//#define FILESTUDIOHEADER_VERSION(master, major, minor, build) ((master & 0xFF) << 24 | (major & 0xFF) << 16 | (minor & 0xFF) << 8 | (build & 0xFF))
//#define FILESTUDIOHEADER_VERSION_UNSUPPORTED                  FILESTUDIOHEADER_VERSION(99, 99, 99, 99)

// Flags fuer den Dateiheader
//#define FILESTUDIOHEADER_FLAG_IGNORE_SHOULD_STUDIO_VERSION        0x00000001
//#define FILESTUDIOHEADER_FLAG_IGNORE_SHOULD_LIBRARY_VERSION       0x00000002
//#define FILESTUDIOHEADER_FLAG_IGNORE_SHOULD_FIRMWARE1_VERSION     0x00000004
//#define FILESTUDIOHEADER_FLAG_IGNORE_SHOULD_FIRMWARE2_VERSION     0x00000008
//#define FILESTUDIOHEADER_FLAG_IGNORE_SHOULD_FIRMWARE3_VERSION     0x00000010

// Flags fuer die Loadroutine Pruefung auf Versionen
//#define FILESTUDIOHEADER_IGNORE_MIN_STUDIO_VERSION            0x00000001
//#define FILESTUDIOHEADER_IGNORE_SHOULD_STUDIO_VERSION         0x00000002
//#define FILESTUDIOHEADER_IGNORE_MIN_LIBRARY_VERSION           0x00000004
//#define FILESTUDIOHEADER_IGNORE_SHOULD_LIBRARY_VERSION        0x00000008

#pragma pack(push, 1)
// Wird der Header verwendet, wird eine Firmware 04.00.xx nicht mehr unterstuetzt!

struct StructFileStudioHeader
{
    uint32_t check;                              ///< Muss immer mit FILESTUDIOHEADER_CHECK belegt werden. 
                                                 ///< Wird herangezogen um zu erkennen ob eine Headerstruktur in der Datei vorliegt.
    uint32_t bugfixSize;                         ///< Groesse der Struktur sizeof_StructFileStudioHeader_1, Version 1. Naeheres siehe Erleuterung zur bugfixSize weiter unten.
                                                 ///< Ist auch gleichzeitig eine Art Datei-Version. Die Struktur kann derzeit maximal (EEP_SIZE - EEP_SIZE_SU) gross werden.
    uint32_t flags;                              ///< Flags fuer z. B. "Gelesen mit DFComDLL, jedoch noch nicht bearbeitet"
    uint32_t studioVersion;                      ///< Studioversion mit der die Datei zuletzt abgespeichert wurde.
                                                 ///< Wurde die Datei ueber die DLL ausgelesen liegt hier keine Angabe vor!
    // Mindestversionen
    uint32_t minStudioVersion;                   ///< Mindestversion des einzusetzenden Studios, um alle enthaltenen Daten zu unterstuetzen.
    uint32_t shouldStudioVersion;                ///< Sollteversion des einzusetzenden Studios, um alle enthaltenen Daten zu unterstuetzen.
    uint32_t minLibraryVersion;                  ///< Mindestversion der zu verwendenden DFComDLL fuer die korrekte Uebertragung der Setupdaten.
    uint32_t shouldLibraryVersion;               ///< Sollteversion der zu verwendenden DFComDLL fuer die korrekte Uebertragung der Setupdaten.
    uint32_t minFirmware1Version;                ///< Mindestversion der zu verwendenden Firmware 04.01.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
    uint32_t shouldFirmware1Version;             ///< Sollteversion der zu verwendenden Firmware 04.01.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
    uint32_t minFirmware2Version;                ///< Mindestversion der zu verwendenden Firmware 04.02.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
    uint32_t shouldFirmware2Version;             ///< Sollteversion der zu verwendenden Firmware 04.02.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
    // CRC der Dateibereiche
    uint32_t crcEep;                             ///< CRC ueber Eep-Bereich.
    uint32_t crcEepExtended;                     ///< CRC ueber EepExtended-Bereich (dort liegt auch Header jedoch mit eigenem CRC)
    uint32_t crcFlash;                           ///< CRC ueber Flash-Bereich.
    uint32_t crcListFile;                        ///< CRC ueber die Listendateiangaben.
    uint32_t crcConfig;                          ///< CRC ueber die Config-Struktur.
    uint32_t crcFlashExtended;                   ///< CRC ueber FlashExtended-Bereich.
    uint32_t crcListEntranceFile;                ///< CRC ueber die Listendateiangaben der ZK1.
    uint32_t crcListEntrance2File;               ///< CRC ueber die Listendateiangaben der ZK2.
    uint32_t crcListExtendedFile;                ///< CRC ueber die Zusaetzlichen Listendateiangaben.
    // Mindestversionen (Hinzugefuegt ab Version 04.03.xx)
    uint32_t bugfixCRC;                          ///< CRC der Headerstruktur Version 1. Naeheres siehe Erleuterung zum bugfixCRC weiter unten.
                                                 ///< Folgende Angaben sind ab VERSION 2 verfuegbar.
    uint32_t size;                               ///< Groesse der Struktur sizeof(StructFileStudioHeader) ab Version 2.
                                                 ///< ACHTUNG: Bei Verwendung wird eine Mindestversion 04.03.xx des Studio und der Library vorausgesetzt.
    uint32_t minFirmware3Version;                ///< Mindestversion der zu verwendenden Firmware 04.03.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
    uint32_t shouldFirmware3Version;             ///< Sollteversion der zu verwendenden Firmware 04.03.xx.xx, um alle gewaehlten Einstellungen zu unterstuetzen.
                                                 //...
    uint32_t crc;                                ///< CRC ueber alle Kopfdaten
};

#define sizeof_StructFileStudioHeader_1     88 // Strukturgroesse der Version 1 ist: 88Byte
#define sizeof_StructFileStudioHeader_2    104 // Strukturgroesse der Version 2 ist: 104Byte

#pragma pack()

/*
Erlaeuterung zum bugfixCRC.
Vorgaengerversionen der Software pruefen zuerst den CRC ueber ihren direkten Strukturzugriff (Version 1). Da sich der CRC bei aktuelleren Versionen jedoch weiter nach hinten
verschiebt, fuehrt dieses dazu, dass es zu einem Head-CRC-Fehler kommt bevor die Pruefung auf die Mindestversion der Library oder des Studios kommt.
Ab dieser Version wird der CRC in den letzten 4 Byte der Struktur auf Grundlage ihrer size Angabe erwartet. Zukuenftig hinzugefuegte Werte in der Struktur werden
unangetastet mitverarbeitet. Muss eine dieser Variablen beachtet werden, muss die Version der Library und des Studios auf die entsprechende Mindestversion gesetzt werden.
Wird zum Beispiel der Parameter minFirmware3Version verwendet, muss die Library und Studio Version auf mindestens 4.3.0.0 gestellt werden, damit dieser Parameter auch
beachtet und verarbeitet wird.
*/

/*
template<std::size_t N, typename Enum>
class FlagSet : public std::bitset<N>
{
public:
    const FlagSet( const FlagSet& other ) : bitset( other.bitset )
    {

    }
};
*/

class FileAES
{
public:
    // Flags fuer den Dateiheader
    enum Enum_headerflagname_ignore_version
    {
        hfiv_studio = 0x00000001,
        hfiv_library = 0x00000002,
        hfiv_firmware1 = 0x00000004,
        hfiv_firmware2 = 0x00000008,
        hfiv_firmware3 = 0x00000010,
    };

    // Flags fuer die Loadroutine Pruefung auf Versionen
    enum Enum_flag_ignore_version
    {
        fiv_min_studio = 0x00000001,
        fiv_should_studio = 0x00000002,
        fiv_min_library = 0x00000004,
        fiv_should_library = 0x00000008
    };

    enum Enum_flag_init
    {
        fi_keepFilePaths = 0x00000001 // Beibehalten der vorhandenen Pfadangaben für Listen und andere verlinkte Dateien.
    };
public:
    FileAES();
    virtual ~FileAES();

    void SetDefault( int32_t flags = 0 );
    void resetFileHeader();

    int32_t load( const std::string &file_path, int32_t *error, Enum_flag_ignore_version flags );
    int32_t save( const std::string &file_path, int32_t *error );
    uint32_t GetFlashSetupSize();
    uint32_t GetExtendedFlashSetupSize();
    uint32_t GetTablesSize( bool lists );  ///< Ermittelt die Groesse der Listen- oder Datensazbeschreibungen.
    int32_t NeededVersion();

    uint16_t endian_reverse( uint16_t val );
    uint32_t endian_reverse( uint32_t val );

    int32_t buildHeader();
    int32_t checkHeader();

    static uint32_t magic_header_number()
    {
        return 0xDFDFDFDF;
    }
    static bool is_magic_header_number( uint32_t number )
    {
        return (number == magic_header_number());
    }
    static uint32_t header_version( uint8_t master, uint8_t major, uint8_t minor, uint8_t build )
    {
        return (((master & 0xFF) << 24) | ((major & 0xFF) << 16) | ((minor & 0xFF) << 8) | (build & 0xFF));
    }
    static uint32_t invalid_header_version()
    {
        return header_version( 99, 99, 99, 99 );
    }
    static bool is_invalid_header_version( uint32_t version )
    {
        return (invalid_header_version() == version);
    }
    inline int32_t IsHeader()
    {
        return is_magic_header_number( m_file_header.check );
    }

    void SetStudioVersion( const char *version )
    {
        m_studioVersion.parse( version );
        m_file_header.studioVersion = header_version( m_studioVersion.master(), m_studioVersion.major(), m_studioVersion.minor(), m_studioVersion.build() );
    }

    // Anpassung auf eventuell benoetigte Mindestversion.
    void NeedStudioVersion( const char *version )
    {
        SoftwarebuildVersion minStudioVersion( m_file_header.minStudioVersion );
        SoftwarebuildVersion minVersion( version );
        if ( minStudioVersion < minVersion )
        {
            m_file_header.minStudioVersion = header_version( minVersion.master(), minVersion.major(), minVersion.minor(), minVersion.build() );
        }
    }

    void ShouldStudioVersion( const char *version )
    {
        SoftwarebuildVersion studioVersion( m_file_header.shouldStudioVersion );
        SoftwarebuildVersion shouldVersion( version );
        if ( studioVersion < shouldVersion )
        {
            m_file_header.shouldStudioVersion = header_version( shouldVersion.master(), shouldVersion.major(), shouldVersion.minor(), shouldVersion.build() );
        }
    }

    void SetLibraryVersion( const char *version )
    {
        m_libraryVersion.parse( version );
    }

    void NeedLibraryVersion( const char *version )
    {
        SoftwarebuildVersion minLibraryVersion( m_file_header.minLibraryVersion );
        SoftwarebuildVersion minVersion( version );
        if ( minLibraryVersion < minVersion )
        {
            m_file_header.minLibraryVersion = header_version( minVersion.master(), minVersion.major(), minVersion.minor(), minVersion.build() );
        }
    }

    void ShouldLibraryVersion( const char *version )
    {
        SoftwarebuildVersion libraryVersion( m_file_header.shouldLibraryVersion );
        SoftwarebuildVersion shouldVersion( version );
        if ( libraryVersion < shouldVersion )
        {
            m_file_header.shouldLibraryVersion = header_version( shouldVersion.master(), shouldVersion.major(), shouldVersion.minor(), shouldVersion.build() );
        }
    }

    void ShouldFirmware1Version( const char *version )
    {
        SoftwarebuildVersion firmwareVersion( m_file_header.shouldFirmware1Version );
        SoftwarebuildVersion shouldVersion( version );
        if ( firmwareVersion < shouldVersion )
        {
            m_file_header.shouldFirmware1Version = header_version( shouldVersion.master(), shouldVersion.major(), shouldVersion.minor(), shouldVersion.build() );
            NeedLibraryVersion( "04.02.02.08" );
            NeedStudioVersion( "04.02.02.08" );
        }
    }

    void NeedFirmware1Version( const char *version )
    {
        SoftwarebuildVersion minFirmwareVersion( m_file_header.minFirmware1Version );
        SoftwarebuildVersion minVersion( version );
        if ( minFirmwareVersion < minVersion )
        {
            m_file_header.minFirmware1Version = header_version( minVersion.master(), minVersion.major(), minVersion.minor(), minVersion.build() );
            NeedLibraryVersion( "04.02.02.08" );
            NeedStudioVersion( "04.02.02.08" );
        }
    }

    void UnsupportFirmware1Version()
    {
        m_file_header.minFirmware1Version = invalid_header_version();
    }

    bool supportFirmware1Version()
    {
        return !is_invalid_header_version( m_file_header.minFirmware1Version );
    }

    void ShouldFirmware2Version( const char *version )
    {
        SoftwarebuildVersion firmwareVersion( m_file_header.shouldFirmware2Version );
        SoftwarebuildVersion shouldVersion( version );
        if ( firmwareVersion < shouldVersion )
        {
            m_file_header.shouldFirmware2Version = header_version( shouldVersion.master(), shouldVersion.major(), shouldVersion.minor(), shouldVersion.build() );
            NeedLibraryVersion( "04.02.02.08" );
            NeedStudioVersion( "04.02.02.08" );
        }
    }

    void NeedFirmware2Version( const char *version )
    {
        SoftwarebuildVersion minFirmwareVersion( m_file_header.minFirmware2Version );
        SoftwarebuildVersion minVersion( version );
        if ( minFirmwareVersion < minVersion )
        {
            m_file_header.minFirmware2Version = header_version( minVersion.master(), minVersion.major(), minVersion.minor(), minVersion.build() );
            NeedLibraryVersion( "04.02.02.08" );
            NeedStudioVersion( "04.02.02.08" );
        }
    }

    void UnsupportFirmware2Version()
    {
        m_file_header.minFirmware2Version = invalid_header_version();
    }

    bool supportFirmware2Version()
    {
        return !is_invalid_header_version( m_file_header.minFirmware2Version );
    }

    void ShouldFirmware3Version( const char *version )
    {
        SoftwarebuildVersion firmwareVersion( m_file_header.shouldFirmware3Version );
        SoftwarebuildVersion shouldVersion( version );
        if ( firmwareVersion < shouldVersion )
        {
            m_file_header.shouldFirmware3Version = header_version( shouldVersion.master(), shouldVersion.major(), shouldVersion.minor(), shouldVersion.build() );
            NeedLibraryVersion( "04.03.00.08" );
            NeedStudioVersion( "04.03.00.08" );
        }
    }

    void NeedFirmware3Version( const char *version )
    {
        SoftwarebuildVersion minFirmwareVersion( m_file_header.minFirmware3Version );
        SoftwarebuildVersion minVersion( version );
        if ( minFirmwareVersion < minVersion )
        {
            m_file_header.minFirmware3Version = header_version( minVersion.master(), minVersion.major(), minVersion.minor(), minVersion.build() );
            NeedLibraryVersion( "04.03.00.08" );
            NeedStudioVersion( "04.03.00.08" );
        }
    }

    void UnsupportFirmware3Version()
    {
        m_file_header.minFirmware3Version = invalid_header_version();
        NeedLibraryVersion( "04.03.00.08" );
        NeedStudioVersion( "04.03.00.08" );
    }

    bool supportFirmware3Version()
    {
        return !is_invalid_header_version( m_file_header.minFirmware3Version );
    }

    void NeedFirmwareVersion( const char *version )
    {
        SoftwarebuildVersion fwVersion( version );
        // Pruefung auf 04.01.xx
        if ( fwVersion >= "04.01.01.00" && fwVersion < "04.02.00.00" )
        {
            // Mindestversion der Firmware 04.01.xx.yy setzen.
            NeedFirmware1Version( version );
        }
        else if ( fwVersion >= "04.02.00.00" && fwVersion < "04.03.00.00" )
        {
            // Mindestversion der Firmware 04.02.xx.yy setzen.
            NeedFirmware2Version( version );
        }
        else if ( fwVersion >= "04.03.00.00" && fwVersion < "04.04.00.00" )
        {
            // Mindestversion der Firmware 04.03.xx.yy setzen.
            NeedFirmware3Version( version );
        }
    }

    void ShouldFirmwareVersion( const char *version )
    {
        SoftwarebuildVersion fwVersion( version );
        // Pruefung auf 04.01.xx
        if ( fwVersion >= "04.01.01.00" && fwVersion < "04.02.00.00" )
        {
            // Empfohlene Firmware 04.01.xx.yy eintragen.
            ShouldFirmware1Version( version );
        }
        else if ( fwVersion >= "04.02.00.00" && fwVersion < "04.03.00.00" )
        {
            // Empfohlene Firmware 04.02.xx.yy pruefen?
            ShouldFirmware2Version( version );
        }
        else if ( fwVersion >= "04.03.00.00" && fwVersion < "04.04.00.00" )
        {
            // Empfohlene Firmware 04.03.xx.yy pruefen?
            ShouldFirmware3Version( version );
        }
    }

    int32_t IsSupportedFirmwareVersion( const char *version )
    {
        SoftwarebuildVersion fwVersion( version );
        // Pruefung auf 04.xx.yy
        if ( (fwVersion >= "04.01.01.00" && fwVersion <= "04.01.09.99")
            || (fwVersion >= "04.02.00.00" && fwVersion <= "04.02.06.99")
            || (fwVersion >= "04.03.00.00" && fwVersion <= "04.03.18.99") )
        {
            return 1;
        }

        return 0;
    }
    int32_t ValidFirmwareVersion( SoftwarebuildVersion &fwVersion );

    // CRC ueber Dateibereiche
    uint32_t GetCrcEep();                ///< CRC ueber Eep-Bereich.
    uint32_t GetCrcEepExtended();        ///< CRC ueber Eep-Bereich.
    uint32_t GetCrcFlash();              ///< CRC ueber Flash-Bereich.
    uint32_t GetCrcListFile();           ///< CRC ueber die Listendateiangaben.
    uint32_t GetCrcConfig();             ///< CRC ueber die Config-Struktur.
    uint32_t GetCrcFlashExtended();      ///< CRC ueber FlashExtended-Bereich.
    uint32_t GetCrcListEntranceFile();   ///< CRC ueber die Listendateiangaben der ZK1.
    uint32_t GetCrcListEntrance2File();  ///< CRC ueber die Listendateiangaben der ZK2.
    uint32_t GetCrcListExtendedFile();   ///< CRC ueber die Zusaetzlichen Listendateiangaben.
    // MD5 ueber Dateibereiche.
    char *GetMD5Header();                    ///< MD5 ueber die Struktur fuer den Dateikopf.
    char *GetMD5Tables( bool lists );        ///< MD5 ueber die Listenbeschreibungen.

    uint32_t writeCString( const std::string &s, std::FILE *fp );
    uint32_t readCString( std::string &s, std::FILE *fp );

    uint32_t crcCString( const std::string &s, uint32_t crc = 0xFFFFFFFF, bool closure = true );
public:
    SoftwarebuildVersion m_studioVersion;
    SoftwarebuildVersion m_libraryVersion;

public:
    std::array<uint8_t, EEP_SIZE_SU> m_eep_mem;
    StructFlashHeader &m_flash_header;
    uint8_t *m_eep;

    std::array<uint8_t, EEP_SIZE - EEP_SIZE_SU> m_file_header_mem;
    StructFileStudioHeader &m_file_header;

    std::array<uint8_t, sizeof( StructConfig )> m_config_mem;
    StructConfig &m_config;

    std::array<uint8_t, FLASH_SIZE_SU + FLASH_SIZE_SU2> m_flash_mem;
    uint8_t *m_flash;

    std::array<uint8_t, EXTENDED_SIZE_SU> m_extended_mem;
    StructExtendedFlash &m_extended_header;
    uint8_t *m_flashExtended;
    uint32_t m_flashSizeSU2;
    uint32_t m_flashMemOffset;

public:
    // Pfadangaben der Listen / Zusatzpfade
    std::array<std::string, 20> m_listFileName;
    std::array<std::string, 4> m_listEntranceFileName;
    std::array<std::string, 20> m_listEntrance2FileName;
    std::array<std::string, 20> m_strExtendedFilePath;
};

#endif
